#include <Wt/WApplication>
#include <Wt/WCssTheme>
#include <Wt/WEnvironment>
#include <Wt/WDefaultLoadingIndicator>
#include <Wt/WLogger>
#include <Wt/WServer>
#include <Wt/WString>
#include <Wt/WVBoxLayout>

#include "wresourcepage.h"


using namespace std;
using namespace Wt;


WApplication *CreateWebApplication(const WEnvironment& env)
{

    WApplication             *app = nullptr;
    WDefaultLoadingIndicator *loadingIndicator = nullptr;
    WVBoxLayout              *siteLayout = nullptr;

    try {
        app = new WApplication(env);
        if (!app->appRoot().empty()) {
            WCssTheme *appTheme = new WCssTheme("default");
            app->setTheme(appTheme);
            app->useStyleSheet("style/basestyles.css");
            app->useStyleSheet("style/condostyles.css");
            app->setTitle("WResource Test");
            WLocale appLocale("");
            appLocale.setDateFormat("dd/MM/yyyy");
            appLocale.setDecimalPoint(string(1, ','));
            appLocale.setGroupSeparator(string(1, '.'));
            app->setLocale(appLocale);
            loadingIndicator = new WDefaultLoadingIndicator();
            loadingIndicator->setMessage("Processing...");
            app->setLoadingIndicator(std::move(loadingIndicator));
            app->styleSheet().addRule("div.Wt-loading", "background-color: green");
            siteLayout = new WVBoxLayout();
            siteLayout->setContentsMargins(0, 0, 0, 0);
            siteLayout->addWidget(new DefaultPage(" Testing WResource use ... "));
            app->root()->setLayout(siteLayout);
        }
    }
    catch (...) {
        delete app;
        delete loadingIndicator;
        delete siteLayout;
    }
    return(app);

}


int main(int argc, char** argv)
{

    int    errorCode = 0;
    string iniFileName, traceFile;

    try {
//        WString::setDefaultEncoding(Local);
        errorCode = WRun(argc, argv, &CreateWebApplication);
    }
    catch (WServer::Exception &e) {
        errorCode = -1;
        Wt::log("info") << " +++ " << e.what();
    }
    catch (exception &e) {
        errorCode = -1;
        string wtf(e.what());
        Wt::log("info") << " +++ exception: " + wtf;
    }
    catch (...) {
        errorCode = -4;
        Wt::log("info") << " +++ server - application terminated. Exceptions had arised ";
    }
    Wt::log("info") << " +++ server - return code: " + to_string(errorCode);
    return(errorCode);

}
