#include <Wt/WApplication.h>
#include <Wt/WCssTheme.h>
#include <Wt/WEnvironment.h>
#include <Wt/WDefaultLoadingIndicator.h>
#include <Wt/WLogger.h>
#include <Wt/WServer.h>
#include <Wt/WString.h>
#include <Wt/WVBoxLayout.h>

#include "wresourcepage.h"


using namespace std;
using namespace Wt;


unique_ptr<WApplication> CreateWebApplication(const WEnvironment& env)
{

    unique_ptr<WApplication>             app = nullptr;
    unique_ptr<WDefaultLoadingIndicator> loadingIndicator = nullptr;
    unique_ptr<WVBoxLayout>              siteLayout = nullptr;

    try {
        app = make_unique<WApplication>(env);
        if (!app->appRoot().empty()) {
            shared_ptr<WCssTheme> appTheme = make_shared<WCssTheme>("default");
            app->setTheme(appTheme);
            app->useStyleSheet("style/basestyles.css");
            app->useStyleSheet("style/condostyles.css");
            app->setTitle("WResource Test");
            WLocale appLocale("");
            appLocale.setDateFormat("dd/MM/yyyy");
            appLocale.setDecimalPoint(string(1, ','));
            appLocale.setGroupSeparator(string(1, '.'));
            app->setLocale(appLocale);
            loadingIndicator = make_unique<WDefaultLoadingIndicator>();
            loadingIndicator->setMessage("Processing...");
            app->setLoadingIndicator(std::move(loadingIndicator));
            app->styleSheet().addRule("div.Wt-loading", "background-color: green");
            siteLayout = make_unique<WVBoxLayout>();
            siteLayout->setContentsMargins(0, 0, 0, 0);
            siteLayout->addWidget(make_unique<DefaultPage>(" Testing WResource use ... "));
            app->root()->setLayout(std::move(siteLayout));
        }
    }
    catch (...) {
        app.reset();
    }
    return(app);

}


int main(int argc, char** argv)
{

    int    errorCode = 0;
    string iniFileName, traceFile;

    try {
        WString::setDefaultEncoding(CharEncoding::Local);
        errorCode = WRun(argc, argv, &CreateWebApplication);
    }
    catch (WServer::Exception &e) {
        errorCode = -1;
        Wt::log("info") << " +++ " << e.what();
    }
    catch (exception &e) {
        errorCode = -1;
        string wtf(e.what());
        Wt::log("info") << " +++ exception: " + wtf;
    }
    catch (...) {
        errorCode = -4;
        Wt::log("info") << " +++ server - application terminated. Exceptions had arised ";
    }
    Wt::log("info") << " +++ server - return code: " + to_string(errorCode);
    return(errorCode);

}
