#include <Wt/WApplication.h>
#include <Wt/WLabel.h>
#include <Wt/WLineEdit.h>
#include <Wt/WPushButton.h>
#include <Wt/WText.h>
#include "wresourcepage.h"


using namespace std;
using namespace Wt;

DefaultPage::DefaultPage(Wt::WString const &defaultMessage) :
dDefaultMessage(nullptr), 
dResourceUrl(nullptr),
dUrlIn(nullptr),
dClearButton(nullptr),
dOutButton(nullptr),
dResourceButton(nullptr),
dWebResource(nullptr)
{

	addWidget(make_unique<WBreak>());
	addWidget(make_unique<WBreak>());
	dDefaultMessage = addWidget(make_unique<WText>());
	dDefaultMessage->setMargin(30, Side::Top | Side::Left);
	dDefaultMessage->setText(defaultMessage);
	addWidget(make_unique<WBreak>());
	addWidget(make_unique<WBreak>());
	dResourceUrl = addWidget(make_unique<WText>());
	dResourceUrl->setMargin(50, Side::Top | Side::Left);
	addWidget(make_unique<WBreak>());
	addWidget(make_unique<WBreak>());
	WLabel *urlLabel = addWidget(make_unique<WLabel>("Url to Jump in   > "));
	urlLabel->setMargin(20, Side::Top | Side::Left);
	dUrlIn = addWidget(make_unique<WLineEdit>());
	dUrlIn->setMargin(5, Side::Left);
	dUrlIn->setTextSize(40);
	dUrlIn->setMaxLength(64);
	urlLabel->setBuddy(dUrlIn);
	addWidget(make_unique<WBreak>());
	addWidget(make_unique<WBreak>());
	dClearButton = addWidget(make_unique<WPushButton>("Clear"));
	dClearButton->setMargin(45, Side::Top | Side::Left);
	dClearButton->clicked().connect(this, &DefaultPage::ClearData);
	dLoadData = addWidget(make_unique<WPushButton>("Load"));
	dLoadData->setMargin(30, Side::Left);
	dLoadData->clicked().connect(this, &DefaultPage::LoadData);
	dResourceButton = addWidget(make_unique<WPushButton>("Activate Resource"));
	dResourceButton->setMargin(30, Side::Left);
	dWebResource = make_shared<WebResourceWidget>();
	WLink theLink(dWebResource);
	theLink.setTarget(LinkTarget::NewWindow);
	dResourceButton->setLink(theLink);
	dResourceButton->setDisabled(true);
	dResourceUrl->setText(dWebResource->GetResourceUrl());
	dOutButton = addWidget(make_unique<WPushButton>("Exit"));
	dOutButton->setMargin(30, Side::Left);
	dOutButton->clicked().connect(this, &DefaultPage::QuitApplication);
	addWidget(make_unique<WBreak>());
	addWidget(make_unique<WBreak>());
	dStatusMessage = addWidget(make_unique<WText>());
	dStatusMessage->setMargin(50, Side::Top | Side::Left);
	dStatusMessage->setStyleClass("staticTitleMsg");

}


void DefaultPage::ClearData(void)
{

	dUrlIn->setText("");
	dStatusMessage->setText("");
	dResourceButton->setDisabled(true);

}


void DefaultPage::LoadData(void)
{

	string theUrl(dUrlIn->text().toUTF8());
	if (theUrl.length()) {
		dStatusMessage->setText("Ready to jump ...");
		dWebResource->SetUrl(theUrl);
		dResourceButton->setDisabled(false);
	}
	else
		dStatusMessage->setText("Need to enter and load the URL value to jump ...");

}


void DefaultPage::QuitApplication(void)
{

	clear();
	WApplication::instance()->quit();

}


DefaultPage::~DefaultPage(void)
{

	clear();
	dWebResource.reset();

}